"use strict";
/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const configurationHelpers_1 = require("../configurationHelpers");
const constants_1 = require("../constants");
const resources_1 = require("../resources");
const resources_2 = require("../resources");
const status_1 = require("../status");
const workspaceHelpers = require("../workspaceFolderHelpers");
const extensionService_1 = require("./extensionService");
const serverProxy_1 = require("./serverProxy");
class BuildService extends extensionService_1.ExtensionService {
    constructor(context, languageServerClient, symbolsService, launchService, editorService) {
        super(context);
        this.languageServerClient = languageServerClient;
        this.symbolsService = symbolsService;
        this.launchService = launchService;
        this.editorService = editorService;
        this.serverProxy = new serverProxy_1.ServerProxy(this.languageServerClient);
    }
    activate() {
        this.registerFolderCommand("al.package", () => this.packageContainer()
            .catch(() => {
            // ignore
        }));
        this.registerFolderCommand("al.publish", () => this.publishContainer()
            .catch(() => {
            // ignore
        }));
        this.registerFolderCommand("al.publishNoDebug", () => this.publishContainer(true)
            .catch(() => {
            // ignore
        }));
        this.registerFolderCommand("al.incrementalPublish", () => this.publishContainer(false, true)
            .catch(() => {
            // ignore
        }));
        this.registerFolderCommand("al.incrementalPublishNoDebug", () => this.publishContainer(true, true)
            .catch(() => {
            // ignore
        }));
        this.registerFolderCommand("al.onlyDebug", () => this.startDebugging(false, false, true)
            .catch(() => {
            // ignore
        }));
        this.registerFolderCommand("al.clearCredentialsCache", () => this.clearCredentialsCache()
            .catch(() => {
            // ignore
        }));
        this.context.workspaceState.update(constants_1.SymbolsCheckedStateKey, false);
    }
    packageContainer(isRad) {
        status_1.outputChannel.show();
        status_1.outputChannel.clear();
        return this.editorService.saveAllDocuments()
            .then(() => this.downloadSymbolsOnce())
            .then(() => this.performBuild(isRad));
    }
    publishContainer(noDebug, isRad) {
        status_1.outputChannel.show();
        status_1.outputChannel.clear();
        const currentWorkspaceFolder = workspaceHelpers.getCurrentWorkspaceFolder();
        if (!currentWorkspaceFolder) {
            return Promise.reject(resources_2.default.noActiveWorkspaceFolder);
        }
        return this.packageContainer(isRad)
            .then(() => this.startDebugging(noDebug, isRad))
            .then(() => {
            // Reload the the Rad file with the state publishing has left it.
            this.reloadRadFile();
        })
            .catch(() => {
            // ignored
        });
    }
    startDebugging(noDebug, isRad, justDebug) {
        status_1.outputChannel.show();
        status_1.outputChannel.clear();
        const currentWorkspaceFolder = workspaceHelpers.getCurrentWorkspaceFolder();
        if (!currentWorkspaceFolder) {
            return Promise.reject(resources_2.default.noActiveWorkspaceFolder);
        }
        return this.launchDebugger(currentWorkspaceFolder, noDebug, isRad, justDebug);
    }
    launchDebugger(currentWorkspaceFolder, noDebug, isRad, justDebug) {
        return this.ensureAuthenticated()
            .then(() => this.launchService.mapToDebugConfiguration(noDebug, isRad, justDebug))
            .then(config => {
            // VSCode not always switches to the debug console, where all the useful message are.
            // Therefore, debug console is activated.
            // If this is the first time debugging is started, do not show the console since debugger selection list has focus.
            //
            // Furthermore, clearing the previous debug console session since otherwise previous session is overlaying current one.
            if (this.launchService.isConfigured()) {
                this.context.executeCommand("workbench.debug.panel.action.clearReplAction");
                this.context.executeCommand("workbench.debug.action.focusRepl");
            }
            return config;
        })
            .then(config => this.context.startDebugging(currentWorkspaceFolder, config))
            .then(() => { });
    }
    downloadSymbolsOnce() {
        let symbolsPromise = null;
        if (!this.context.workspaceState.get(constants_1.SymbolsCheckedStateKey)) {
            symbolsPromise = this.symbolsService.checkSymbols()
                .then(result => {
                if (!result) {
                    return this.symbolsService.downloadSymbols(true);
                }
                return null;
            })
                .catch(() => {
                // ignore symbol errors to get compilation errors in the console as well
            });
        }
        else {
            symbolsPromise = Promise.resolve();
        }
        return symbolsPromise;
    }
    performBuild(isRad) {
        return new Promise((resolve, reject) => {
            const workspaceRoot = workspaceHelpers.getCurrentWorkspaceFolderPath();
            if (workspaceRoot) {
                const args = ['-project:"' + workspaceRoot + '"'];
                for (const arg of this.parseConfigurationArguments()) {
                    args.push(arg);
                }
                this.languageServerClient.sendRequest(constants_1.AlCreatePackageRequest, { projectDir: workspaceRoot, args, isRad: isRad })
                    .then((response) => {
                    status_1.outputChannel.appendLine(response.success ? resources_1.default.packageCreatedMessage : resources_1.default.packageNotCreatedError);
                    if (response.success) {
                        resolve();
                        this.context.workspaceState.update(constants_1.SymbolsCheckedStateKey, true);
                    }
                    else {
                        reject();
                    }
                }, reject);
            }
            else {
                reject(resources_2.default.noActiveWorkspaceFolder);
            }
        });
    }
    parseConfigurationArguments() {
        const args = [];
        const alConfig = vscode.workspace.getConfiguration(constants_1.AlLanguageId);
        const compilationOptions = alConfig.get("CompilationOptions");
        if (compilationOptions instanceof Object) {
            if (compilationOptions.generateReportLayout !== undefined) {
                args.push("-generateReportLayout" + (compilationOptions.generateReportLayout ? "+" : "-"));
            }
            if (compilationOptions.parallel !== undefined) {
                args.push("-parallel" + (compilationOptions.parallel ? "+" : "-"));
            }
        }
        return args;
    }
    reloadRadFile() {
        return new Promise((resolve, reject) => {
            return this.languageServerClient.sendRequest(constants_1.AlReloadRadFile, {});
        });
    }
    /**
     * Ensures that the user is authenticated in case if publishing is the first action which requires server communication.
     * Note that if multiple configurations are specified, the extension code cannot know which one will picked up by VSCode,
     * therefore in this case it is assumed that the user is already authenticated (from the previous actions).
     */
    ensureAuthenticated() {
        const configurations = this.launchService.getConfigurations();
        if (configurations.length !== 1) {
            return Promise.resolve();
        }
        const params = configurationHelpers_1.getAlParams();
        return this.serverProxy.ensureAuthenticated(configurations[0], params);
    }
    clearCredentialsCache() {
        this.languageServerClient.sendRequest(constants_1.AlClearCredentialsCacheRequest, {});
        return Promise.resolve();
    }
}
exports.BuildService = BuildService;
//# sourceMappingURL=buildService.js.map