"use strict";
/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const uuid = require("node-uuid");
const path = require("path");
const vscode = require("vscode");
const workspaceHelpers = require("../workspaceFolderHelpers");
const clipboard_1 = require("../clipboard");
const constants_1 = require("../constants");
const fsHelpers_1 = require("../fsHelpers");
const resources_1 = require("../resources");
const resources_2 = require("../resources");
const vscodeHelpers_1 = require("../vscodeHelpers");
const extensionService_1 = require("./extensionService");
/**
 * The class for generating artifacts such as app.json and other content files.
 */
class ContentService extends extensionService_1.ExtensionService {
    constructor(context, languageServerClient) {
        super(context);
        this.languageServerClient = languageServerClient;
        if (this.languageServerClient) {
            this.languageServerClient.onReady()
                .then(() => this.languageServerClient.onNotification(constants_1.AlManifestMissingEvent, () => this.onManifestMissingEvent()))
                .then(() => this.languageServerClient.onNotification(constants_1.AlDeviceLoginEvent, object => this.onDeviceLoginEvent(object)));
        }
    }
    activate() {
        const disposable = vscode.commands.registerCommand("al.generateManifest", () => this.generateAppJson(workspaceHelpers.getCurrentWorkspaceFolderPath(), true));
        this.registerForDisposal(disposable);
    }
    generateAppJson(rootPath, show = false) {
        if (!rootPath) {
            return Promise.reject(resources_2.default.noActiveWorkspaceFolder);
        }
        const appJson = path.join(rootPath, constants_1.AlProjectFileName);
        const projectName = path.basename(rootPath);
        let result = fsHelpers_1.existsFileAsync(appJson)
            .then(exists => {
            if (exists) {
                return appJson;
            }
            const json = {
                "id": uuid.v4(),
                "name": projectName,
                "publisher": "Default publisher",
                "version": "1.0.0.0",
                "brief": "",
                "description": "",
                "privacyStatement": "",
                "EULA": "",
                "help": "",
                "url": "",
                "logo": "",
                "dependencies": [],
                "screenshots": [],
                "platform": "14.0.0.0",
                "application": "14.0.0.0",
                "idRanges": [
                    {
                        "from": 50100,
                        "to": 50149
                    }
                ],
                "contextSensitiveHelpUrl": "https://" + projectName.replace(/\s/g, "") + ".com/help/",
                "showMyCode": true,
                "runtime": "3.2"
            };
            return fsHelpers_1.writeTextFileAsync(appJson, JSON.stringify(json, null, "  "))
                .then(() => appJson);
        });
        if (show) {
            result = result.then(resultAppJson => vscodeHelpers_1.openTextDocument(resultAppJson));
        }
        return result;
    }
    generateDefaultAlFile(rootPath) {
        // samples could be downloaded from our github here
        const filePath = path.join(rootPath, constants_1.DefaultAlFileName);
        return fsHelpers_1.existsFileAsync(filePath)
            .then(exists => {
            if (exists) {
                return filePath;
            }
            const contents = `// Welcome to your new AL extension.
// Remember that object names and IDs should be unique across all extensions.
// AL snippets start with t*, like tpageext - give them a try and happy coding!

pageextension 50100 CustomerListExt extends "Customer List"
{
    trigger OnOpenPage();
    begin
        Message('App published: Hello world');
    end;
}`;
            return fsHelpers_1.writeTextFileAsync(filePath, contents)
                .then(() => filePath);
        });
    }
    onManifestMissingEvent() {
        vscode.window.showWarningMessage(resources_1.default.manifestMissingWarning, resources_1.default.generateManifestAction)
            .then(choice => {
            if (choice === resources_1.default.generateManifestAction) {
                this.generateAppJson(workspaceHelpers.getCurrentWorkspaceFolderPath(), true);
            }
        });
    }
    onDeviceLoginEvent(message) {
        vscode.window.showInformationMessage(message.message, resources_1.default.copyAndOpen)
            .then(choice => {
            if (choice === resources_1.default.copyAndOpen) {
                clipboard_1.copyTextToClipboard(message.token);
                vscode.commands.executeCommand("vscode.open", vscode.Uri.parse(message.uri));
            }
        });
    }
}
exports.ContentService = ContentService;
//# sourceMappingURL=contentService.js.map