"use strict";
/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const util = require("util");
const vscode = require("vscode");
const configurationHelpers_1 = require("../configurationHelpers");
const constants_1 = require("../constants");
const fsHelpers_1 = require("../fsHelpers");
const resources_1 = require("../resources");
const resources_2 = require("../resources");
const vscodeHelpers_1 = require("../vscodeHelpers");
const workspaceHelpers = require("../workspaceFolderHelpers");
const alDebugConfigurationProvider_1 = require("./alDebugConfigurationProvider");
const extensionService_1 = require("./extensionService");
const LaunchFilePath = ".vscode/launch.json";
const LaunchFileVersion = "0.2.0";
const ConfigHeader = {
    "type": constants_1.AlLanguageId,
    "request": "launch"
};
const CloudConfig = {
    name: "Microsoft cloud sandbox",
    startupObjectId: 22,
    startupObjectType: "Page",
    breakOnError: true,
    launchBrowser: true
};
const LocalConfig = {
    name: "Your own server",
    server: "http://localhost",
    serverInstance: "BC130",
    authentication: "UserPassword",
    startupObjectId: 22,
    startupObjectType: "Page",
    breakOnError: true,
    launchBrowser: true
};
/**
 * The service responsible for working with launch.json file - generating it, reading configurations, etc.
 */
class LaunchService extends extensionService_1.ExtensionService {
    constructor(context) {
        super(context);
    }
    activate() {
        let disposable = vscode.debug.registerDebugConfigurationProvider(constants_1.AlLanguageId, new alDebugConfigurationProvider_1.AlDebugConfigurationProvider(this));
        this.registerForDisposal(disposable);
        disposable = vscode.commands.registerCommand("al.resolveDebugCommand", args => this.resolveDebugCommand(args).catch(reason => {
            if (reason) {
                vscode.window.showErrorMessage(reason);
            }
        }));
        this.registerForDisposal(disposable);
    }
    getLaunchConfiguration() {
        const configurations = this.getConfigurations();
        if (configurations.length === 0) {
            this.generateLaunchJson()
                .then(success => {
                if (success) {
                    this.openLaunchJson();
                }
            });
            return Promise.reject(new Error(resources_2.default.noLaunchFileWarning));
        }
        let configurationChosen = null;
        if (configurations.length === 1) {
            configurationChosen = Promise.resolve(configurations[0]);
        }
        else {
            configurationChosen = this.chooseLaunchConfiguration(configurations);
        }
        return Promise.resolve(configurationChosen);
    }
    mapToDebugConfiguration(noDebug, isRad, justDebug) {
        return this.getLaunchConfiguration().then(configuration => {
            if (justDebug) {
                noDebug = false;
                isRad = false;
            }
            if (noDebug) {
                justDebug = false;
            }
            const debugConfig = {
                name: configuration.name,
                type: constants_1.AlLanguageId,
                request: "launch",
                noDebug: noDebug,
                isRad: isRad,
                justDebug: justDebug,
                authentication: configuration.authentication,
                port: configuration.port,
                schemaUpdateMode: configuration.schemaUpdateMode,
                server: configuration.server,
                serverinstance: configuration.serverInstance,
                startupObjectId: configuration.startupObjectId,
                startupObjectType: configuration.startupObjectType,
                tenant: configuration.tenant,
                applicationFamily: configuration.applicationFamily,
                breakOnError: configuration.breakOnError,
                breakOnRecordWrite: configuration.breakOnRecordWrite,
                skipCodeunit1: configuration.skipCodeunit1,
                launchBrowser: configuration.launchBrowser,
                sandboxName: configuration.sandboxName,
            };
            return Promise.resolve(debugConfig);
        });
    }
    isConfigured() {
        if (!vscode.workspace.workspaceFolders) {
            return false;
        }
        const w = this.getLaunchFilePath();
        if (!w) {
            return false;
        }
        return fs.existsSync(w);
    }
    isDirty() {
        const w = this.getLaunchFilePath();
        if (!w) {
            return false;
        }
        for (const doc of vscode.workspace.textDocuments) {
            if (doc.isDirty && doc.fileName === w) {
                return true;
            }
        }
        return false;
    }
    getConfigurations() {
        if (!this.isConfigured()) {
            return [];
        }
        const w = this.getLaunchFilePath();
        if (!w) {
            return [];
        }
        const config = vscode.workspace.getConfiguration("launch", vscode.Uri.file(w));
        const configurations = config.get("configurations");
        if (!Array.isArray(configurations)) {
            vscode.window.showErrorMessage(util.format(resources_2.default.jsonMalformedError, LaunchFilePath));
            return [];
        }
        const alParams = configurationHelpers_1.getAlParams();
        const alConfigurations = configurations.filter(x => x.type === constants_1.AlLanguageId);
        for (const alConfig of alConfigurations) {
            alConfig.environment = alParams.env;
        }
        return alConfigurations;
    }
    generateLaunchJson() {
        const filePath = this.getLaunchFilePath();
        if (!filePath) {
            return Promise.reject(resources_1.default.noActiveWorkspaceFolder);
        }
        return fsHelpers_1.existsFileAsync(filePath)
            .then(exists => {
            if (exists) {
                return filePath;
            }
            const dir = path.dirname(filePath);
            if (!fs.existsSync(dir)) {
                try {
                    fs.mkdirSync(dir);
                }
                catch (e) {
                    return null;
                }
            }
            return this.getInitialFileContent()
                .then(config => this.tryUpdate(config))
                .then(() => filePath);
        });
    }
    openLaunchJson() {
        const w = this.getLaunchFilePath();
        if (!w) {
            Promise.reject(resources_1.default.noActiveWorkspaceFolder);
        }
        return vscodeHelpers_1.openTextDocument(w);
    }
    formatLaunchConfiguration(config) {
        if (config.server) {
            return util.format(resources_2.default.serverFormat, config.server, config.serverInstance, config.tenant || "default");
        }
        return resources_2.default.cloudServer;
    }
    getInitialConfigurations() {
        return this.chooseLaunchConfiguration([CloudConfig, LocalConfig])
            .catch(() => LocalConfig)
            .then(config => [Object.assign({}, ConfigHeader, config)]);
    }
    chooseLaunchConfiguration(configurations) {
        return new Promise((resolve, reject) => {
            const items = configurations.map(x => ({
                label: x.name,
                detail: this.formatLaunchConfiguration(x),
                description: ""
            }));
            const options = ({
                placeHolder: resources_2.default.chooseServerPrompt,
                matchOnDetail: true,
                ignoreFocusOut: true
            });
            vscode.window.showQuickPick(items, options).then(item => {
                if (item) {
                    const i = items.indexOf(item);
                    if (i !== -1) {
                        resolve(configurations[i]);
                    }
                    else {
                        reject();
                    }
                }
                else {
                    reject();
                }
            });
        });
    }
    getLaunchFilePath() {
        const currentFolder = workspaceHelpers.getCurrentWorkspaceFolderPath();
        if (currentFolder) {
            return path.join(currentFolder, LaunchFilePath);
        }
    }
    resolveDebugCommand(eventArgs) {
        const activeWs = workspaceHelpers.getCurrentWorkspaceFolder();
        if (!activeWs) {
            return Promise.reject(resources_1.default.noActiveWorkspaceFolder);
        }
        if (eventArgs) {
            const launchWs = vscode.workspace.getWorkspaceFolder(vscode.Uri.parse(eventArgs));
            if (activeWs !== launchWs) {
                return Promise.reject(util.format(resources_2.default.debugConfigrationMismatch, launchWs.name, activeWs.name, activeWs.name));
            }
        }
        const args = ["/startDebugging"];
        const alConfig = vscode.workspace.getConfiguration(constants_1.AlLanguageId);
        this.addArg(args, alConfig, "env");
        this.addArg(args, alConfig, "browser");
        this.addArg(args, alConfig, "incognito");
        this.addArg(args, alConfig, "editorServicesLogLevel", "logLevel");
        if (alConfig.get("debug")) {
            args.push("/waitForDebugger");
        }
        args.push("/projectRoot:" + activeWs.uri.fsPath);
        const serverPath = this.getServerPath();
        return Promise.resolve({
            command: serverPath,
            args
        });
    }
    addArg(args, config, configSettingName, commandLineSettingName = configSettingName) {
        const setting = config.get(configSettingName);
        if (setting) {
            args.push(`/${commandLineSettingName}:${setting}`);
        }
    }
    getInitialFileContent() {
        return this.getInitialConfigurations()
            .then(configs => {
            const initialConfigurations = {
                version: LaunchFileVersion,
                configurations: configs
            };
            return initialConfigurations;
        });
    }
    getConfigurationTarget() {
        return vscode.workspace.workspaceFolders.length > 1 ? vscode.ConfigurationTarget.WorkspaceFolder : vscode.ConfigurationTarget.Workspace;
    }
    tryUpdate(config) {
        const w = workspaceHelpers.getCurrentWorkspaceFolderPath();
        const configuration = vscode.workspace.getConfiguration(null, vscode.Uri.file(w));
        if (vscode.workspace.workspaceFolders.length !== 1) {
            return configuration.update("launch", config, this.getConfigurationTarget());
        }
        // weird scenario when we have one folder and we do not know if it is a workspacefolder.
        return configuration.update("launch", config, vscode.ConfigurationTarget.Workspace)
            .then(() => { }, (reason) => configuration.update("launch", config, vscode.ConfigurationTarget.WorkspaceFolder));
    }
}
exports.LaunchService = LaunchService;
//# sourceMappingURL=launchService.js.map