"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
const fs = require("fs");
const vscode = require("vscode");
const constants_1 = require("./constants");
function getWorkspacePath(path) {
    const uri = vscode.Uri.file(path);
    const workspaceFolder = vscode.workspace.getWorkspaceFolder(uri);
    if (!workspaceFolder) {
        return vscode.workspace.workspaceFolders.length > 0 ? undefined : vscode.workspace.workspaceFolders[0].uri.fsPath;
    }
    return workspaceFolder.uri.fsPath;
}
exports.getWorkspacePath = getWorkspacePath;
function getCurrentWorkspaceFolderPath() {
    const currentWorkspaceFolder = getCurrentWorkspaceFolder();
    if (currentWorkspaceFolder) {
        return currentWorkspaceFolder.uri.fsPath;
    }
    return undefined;
}
exports.getCurrentWorkspaceFolderPath = getCurrentWorkspaceFolderPath;
function getCurrentWorkspaceFolder() {
    if (!Array.isArray(vscode.workspace.workspaceFolders) || vscode.workspace.workspaceFolders.length === 0) {
        return undefined;
    }
    // one workspace folder, this is the old folder/rootpath scenario.
    if (vscode.workspace.workspaceFolders.length === 1) {
        return vscode.workspace.workspaceFolders[0];
    }
    // in VsCode there is no notion of active project.
    // we have therefore to recurse to some logic to find which is the "active workspace"
    // for this
    // 1. We try the active text editor
    // 2. We try the visible text editor which has an 'al' file, a 'json' file or any text file that exists.
    const activeEditor = vscode.window.activeTextEditor;
    if (activeEditor) {
        if (activeEditor.document && isPossibleProjectFile(activeEditor.document)) {
            return getWorkspaceFolderFromPath(activeEditor.document.fileName);
        }
    }
    const visibleTextEditors = vscode.window.visibleTextEditors;
    if (visibleTextEditors && visibleTextEditors.length > 0) {
        for (const e of visibleTextEditors) {
            if (isPossibleProjectFile(e.document)) {
                return getWorkspaceFolderFromPath(e.document.fileName);
            }
        }
    }
    return undefined;
}
exports.getCurrentWorkspaceFolder = getCurrentWorkspaceFolder;
function isPossibleProjectFile(document) {
    return (document.languageId === constants_1.AlLanguageId || document.languageId === "json" ||
        document.uri.scheme === "file") && fs.existsSync(document.fileName);
}
exports.isPossibleProjectFile = isPossibleProjectFile;
function getWorkspaceFolderFromPath(path) {
    const uri = vscode.Uri.file(path);
    return vscode.workspace.getWorkspaceFolder(uri);
}
//# sourceMappingURL=workspaceFolderHelpers.js.map